{% if messages %}
  <div class="django-messages">
    {% for message in messages %}
      {% if message.tags == 'error' %}
        <div class="alert alert-danger">{{ message }}</div>
      {% else %}
        <div class="alert alert-{{ message.tags }}">{{ message }}</div>
      {% endif %}
    {% endfor %}
  </div>
{% endif %}