{% load staticfiles %}

<!DOCTYPE html>
<html>
  <head>
    <title>{% block title %}{% endblock %}</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    
    <!-- Bootstrap -->
    <link href="{% static 'yaccounts/libs/bootstrap-3.0.3/css/bootstrap.min.css' %}" rel="stylesheet">
    
    <!-- Font Awesome -->
    <link href="{% static 'yaccounts/libs/font-awesome-4.0.3/css/font-awesome.min.css' %}" rel="stylesheet">

    <!-- Magnific Popup -->
    <link href="{% static 'yaccounts/libs/magnific-popup-0.9.2/magnific-popup.css' %}" rel="stylesheet">

    <!-- Salsa Form Validator -->
    <link href="{% static 'yaccounts/libs/salsa-0.9/salsa.css' %}" rel="stylesheet">    
    
    <!-- Yaccounts -->
    <link href="{% static 'yaccounts/css/style.css' %}" rel="stylesheet">

    <!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->
    <!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
    <!--[if lt IE 9]>
      <script src="https://oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js"></script>
      <script src="https://oss.maxcdn.com/libs/respond.js/1.3.0/respond.min.js"></script>
    <![endif]-->

    {% block head %}{% endblock %}
  </head>
  
  <body>

    {% include 'yaccounts/navbar.html' %}

    {% block body %}{% endblock %}

    {% include 'yaccounts/footer.html' %}

    <!-- jQuery (necessary for Bootstrap's JavaScript plugins) -->
    <script src="{% static 'yaccounts/libs/jquery-1.10.2.min.js' %}"></script>
    
    <!-- Include all compiled plugins (below), or include individual files as needed -->
    <script src="{% static 'yaccounts/libs/bootstrap-3.0.3/js/bootstrap.min.js' %}"></script>
    
    <!-- Magnific Popup -->
    <script src="{% static 'yaccounts/libs/magnific-popup-0.9.2/jquery.magnific-popup.min.js' %}"></script>

    <!-- Salsa Form Validator -->
    <script src="{% static 'yaccounts/libs/salsa-0.9/salsa.js' %}"></script>
    
    <!-- Begin CSRF stuff for jQuery -->
    <script>
      function getCookie(name) {
          var cookieValue = null;
          if (document.cookie && document.cookie != '') {
              var cookies = document.cookie.split(';');
              for (var i = 0; i < cookies.length; i++) {
                  var cookie = jQuery.trim(cookies[i]);
                  // Does this cookie string begin with the name we want?
                  if (cookie.substring(0, name.length + 1) == (name + '=')) {
                      cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
                      break;
                  }
              }
          }
          return cookieValue;
      }
      var csrftoken = getCookie('csrftoken');
      function csrfSafeMethod(method) {
          // these HTTP methods do not require CSRF protection
          return (/^(GET|HEAD|OPTIONS|TRACE)$/.test(method));
      }
      function sameOrigin(url) {
          // test that a given url is a same-origin URL
          // url could be relative or scheme relative or absolute
          var host = document.location.host; // host + port
          var protocol = document.location.protocol;
          var sr_origin = '//' + host;
          var origin = protocol + sr_origin;
          // Allow absolute or scheme relative URLs to same origin
          return (url == origin || url.slice(0, origin.length + 1) == origin + '/') ||
              (url == sr_origin || url.slice(0, sr_origin.length + 1) == sr_origin + '/') ||
              // or any other URL that isn't scheme relative or absolute i.e relative.
              !(/^(\/\/|http:|https:).*/.test(url));
      }
      $.ajaxSetup({
          beforeSend: function(xhr, settings) {
              if (!csrfSafeMethod(settings.type) && sameOrigin(settings.url)) {
                  // Send the token to same-origin, relative URLs only.
                  // Send the token only if the method warrants CSRF protection
                  // Using the CSRFToken value acquired earlier
                  xhr.setRequestHeader("X-CSRFToken", csrftoken);
              }
          }
      });
    </script>
    <!-- End CSRF stuff for jQuery -->

    {% block javascript %}{% endblock %}
  </body>
</html>