.container {
  margin-top: 15px;
}

/*
 * Django Messages
 */
.django-messages {
  padding-left: 30px;
  padding-right: 30px;
}
.django-messages .alert:last-child {
  margin-bottom: 0;
}

/*
 * Account login/create/reset
 */
.form-signin, .form-signup
{
  max-width: 330px;
  padding: 15px;
  margin: 0 auto;
}
.form-signin .form-signin-heading, .form-signin .checkbox
{
  margin-bottom: 10px;
}
.form-signin .checkbox
{
  font-weight: normal;
}
.form-signin .form-control, .form-signup .form-control
{
  position: relative;
  font-size: 16px;
  height: auto;
  padding: 10px;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.form-signin .form-control:focus, .form-signup .form-control:focus
{
  z-index: 2;
}
.form-signin input[type="text"]
{
  margin-bottom: -1px;
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}
.form-signin input[type="password"]
{
  margin-bottom: 10px;
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}
.account-wall
{
  margin-top: 20px;
  padding: 40px 0px 20px 0px;
  background-color: #f7f7f7;
  -moz-box-shadow: 0px 2px 2px rgba(0, 0, 0, 0.3);
  -webkit-box-shadow: 0px 2px 2px rgba(0, 0, 0, 0.3);
  box-shadow: 0px 2px 2px rgba(0, 0, 0, 0.3);
}
.login-title
{
  color: #555;
  font-size: 18px;
  font-weight: 400;
  display: block;
}
.profile-img
{
  width: 96px;
  height: 96px;
  margin: 0 auto 10px;
  display: block;
  -moz-border-radius: 50%;
  -webkit-border-radius: 50%;
  border-radius: 50%;
}
.need-help
{
  margin-top: 10px;
}
.new-account
{
  display: block;
  margin-top: 10px;
}
.login-or {
  position: relative;
  font-size: 18px;
  color: #aaa;
  margin-top: 24px;
          margin-bottom: 10px;
  padding-top: 10px;
  padding-bottom: 10px;
  padding-left: 30px;
  padding-right: 30px;
}
.span-or {
  display: block;
  position: absolute;
  left: 50%;
  top: -2px;
  margin-left: -25px;
  background-color: #f7f7f7;
  width: 50px;
  text-align: center;
}
.hr-or {
  background-color: #cdcdcd;
  height: 1px;
  margin-top: 0px !important;
  margin-bottom: 0px !important;
}
.social-box{
  margin-top: 25px;
  padding-left: 30px;
  padding-right: 30px;
}
.social-box a{
  font-weight:bold;
  font-size:18px;
  padding:8px;
}
.social-box a i{
  font-weight:bold;
  font-size:20px;
}

/*
 * Account Settings
 */
.form-settings {
  max-width: 300px;
  padding: 19px 29px 29px;
  margin: 0 auto 20px;
}
.form-settings {
  margin-bottom: 0;
}
.form-settings input[type="text"],
.form-settings input[type="password"] {
  font-size: 16px;
  height: auto;
  margin-bottom: 15px;
  padding: 7px 9px;
}
#crop-image-window input[type="file"] {
  display: inline;
}
.white-popup {
  position: relative;
  background: #FFF;
  padding: 20px;
  width: auto;
  max-width: 300px;
  margin: 20px auto;
}
#api table td {
  padding-bottom: 1.5em;
}
.active-indicator {
  background-color: #46a800;
  -webkit-border-radius: 5px;
  display: inline-block;
  height: 10px;
  line-height: 10px;
  padding: 0;
  width: 10px;
}