<html>
    <head>
    </head>
    <body>
    	<h1>Hello, {{ name }}!</h1>
      <p>
        In order to update your account's email address, please click
        <a href="{{ confirmation_link }}">here</a>.
      </p>
    </body>
</html>