<html>
    <head>
    </head>
    <body>
    	<h1>Hello, {{ name }}!</h1>
      <p>
        Thank you for creating an account with us.
      </p>
      <p>
        To activate it, we just need you to confirm that this is your email address by clicking 
        <a href="{{ activation_link }}">here</a>.
      </p>
    </body>
</html>