/*input.salsa-success, textarea.salsa-success {
  color: #468847 !important;
  background-color: #DFF0D8 !important;
  border: 1px solid #D6E9C6 !important;
}*/
input.salsa-error, textarea.salsa-error, select.salsa-error {
  color: #B94A48 !important;
  background-color: #F2DEDE !important;
  border: 1px solid #EED3D7 !important;
}
ul.salsa-error-list {
    font-size: 11px;
    color: #B94A48 !important;
    margin: 2px;
    list-style-type:none;
    padding-left: 0;
}
ul.salsa-error-list li {
    line-height: 11px;
}