<html>
    <head>
    </head>
    <body>
    	<h1>Hello, {{ name }}!</h1>
      <p>
        To reset your password click <a href="{{ reset_link }}">here</a>.
      </p>
      <p>
        If you didn't request a password reset, just ignore this email.
      </p>
    </body>
</html>